/* ============================================================== */
/*                             C PRELUDE                          */
/* ============================================================== */

%{
#include <stdlib.h>
#include <stdio.h>

int		line_number	= 1;	/* number of current source line */
extern int	yylex();	/* lexical analyzer generated from lex.l */
extern char	*yytext;	/* last token, defined in lex.l  */

void yyerror(char *s)
  { fprintf(stderr, 
	    "d parser:%d: syntax error. Last token was \"%s\"\n",
	    line_number, yytext);
    exit(1);
  }
void error(char *s)
  { fprintf(stderr, 
	    "d parser:%d: %s\n", line_number, s);
    exit(1);
  }
%}

/* ============================================================== */
/*                     TYPES OF NON-TERMINALS                     */         
/* ============================================================== */

%union
{
  int integer;
  char* id_string;
}


/* ============================================================== */
/*                       TOKENS DECLARATION
/* ============================================================== */

/* dlimiteurs */
%token TK_LPAR TK_RPAR
%token TK_COMMA TK_SEMI

/* oprations boolennes */
%token TK_TRUE TK_FALSE

/* oprations mixtes */
%token TK_GREATER TK_EQUAL TK_NEQUAL

/* oprations arithmtiques */
%token TK_MUL TK_DIV TK_PLUS TK_MINUS
%token TK_NUM
%token TK_ID

/* dclarations */
%token TK_INT

/* controle */
%token TK_ASSIGN TK_IF TK_THEN TK_ELSE TK_FI TK_WHILE TK_DO TK_DONE
%token TK_RETURN

/* fonctions */
%token TK_FUNCTION TK_BEGIN TK_END

/* associativit */
%left TK_PLUS TK_MINUS
%left TK_MUL TK_DIV

%type<integer> TK_NUM
%type<id_string> TK_ID

/* axiom */
%start prog

%%

/* ============================================================== */
/*               GRAMMAR ITSELF WITH SEMANTIC ACTIONS             */
/* ============================================================== */

/* -------------------------------------------------------------- */
/*                  1/ Expressions (e.g. 2*x+y)                   */
/* -------------------------------------------------------------- */

//ETF (sub-)grammar

e_expr: 
  e_expr TK_PLUS t_expr 
| e_expr TK_MINUS t_expr 
| t_expr 
;

t_expr: 
  t_expr TK_MUL f_expr
| f_expr
;

f_expr: 
  TK_NUM
| TK_ID 
| TK_ID TK_LPAR caller_arg_list TK_RPAR
| TK_LPAR e_expr TK_RPAR 
;

expr: 
  e_expr
;

/* -------------------------------------------------------------- */
/*                2/ Conditions (e.g. 2*x+y != 0)                 */
/* -------------------------------------------------------------- */

/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
/* TODO: ajouter || et &&                                         */

test: 
  expr TK_GREATER expr 
| expr TK_EQUAL expr
| expr TK_NEQUAL expr
| TK_LPAR test TK_RPAR
;

/* -------------------------------------------------------------- */
/*                 3/ Statements (e.g. if, while)                 */
/* -------------------------------------------------------------- */


stmt:

/* ---                ASSIGNMENT (e.g. x := e;)                -- */

  TK_ID TK_ASSIGN expr                             


/* ---                    CONDITIONAL (IF)                     -- */

| TK_IF test TK_THEN stmt_list TK_ELSE stmt_list TK_FI

/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
/* TODO: ajouter le WHILE                                         */


/* ---                       RETURN                            -- */

| TK_RETURN expr
;

stmt_list:
| stmt_list stmt
;

/* -------------------------------------------------------------- */
/*                 4/ Declarations (e.g. int x,y;)                */
/* -------------------------------------------------------------- */

/* Function arguments  (e.g. fact(int n)) */

arg_list:
  TK_ID
| arg_list TK_COMMA TK_ID
;

decl_args: 
| TK_INT arg_list
;

/* Local variables (e.g. int tmp1,tmp2,tmp3) */

/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
/* TODO: afficher les variables                                   */

local_var_list:
  TK_ID
| local_var_list TK_COMMA TK_ID
;

decl_local_vars: 
| TK_INT local_var_list
;

/* -------------------------------------------------------------- */
/*                 5/ Functions                                   */
/* -------------------------------------------------------------- */

caller_arg_list:
| expr 
| caller_arg_list TK_COMMA expr
;


/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
/* TODO: definir 'function'                                       */

function: decl_local_vars stmt /* TO FIX */

function_list:
| function_list function
;


/* -------------------------------------------------------------- */
/*      6/ *Main production* (program=declarations+statements)    */
/* -------------------------------------------------------------- */

prog: function_list function
;
